﻿//
// 引数で指定された文字列に従い、キーボード入力をシミュレートします。

using System;
using System.Collections.Generic;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading;

namespace keysimcs
{
    // Win32APIを呼び出すためのクラス
    public class win32api
    {
        [DllImport("user32.dll")]
        public static extern uint keybd_event(byte bVk, byte bScan, uint dwFlags, UIntPtr dwExtraInfo);
    }

    class keysimcs
    {
        public static int Main(string[] args)
        {
            // 入力対象のプログラムをアクティブにするまでの待ち時間（ミリ秒単位）
            int iSleepTime = 5000;
            // キーの取りこぼしを防ぐための、各入力間の待機時間（ミリ秒単位）
            int iWaitTime = 10;
            // ※1ミリ秒＝0.001秒のこと

            if (args.Length == 0) {
                // 引数が指定されていない場合、使用方法を表示
                Console.WriteLine("キーボード入力をシミュレートしたい文字列を引数に指定してください。");
                Console.WriteLine("");
                Console.WriteLine("keysimcs <文字列> [<待機時間> [<文字間ウェイト>]]");
                Console.WriteLine("");
                Console.WriteLine("文字列に指定可能な文字は、英大文字・数字・スペース・タブです。");
                Console.WriteLine("（英小文字は無視されますのでご注意下さい）");
                Console.WriteLine("スペース・タブを指定する場合は、文字列全体をダブルコーテーションで囲います。");
                Console.WriteLine("待機時間は、入力開始までの待ち時間をミリ秒単位で指定します(1以上)。");
                Console.WriteLine("文字間ウェイトは、入力文字間のウェイト時間をミリ秒単位で指定します(1以上)。");
                return 1;
            }

            // 待機時間取得
            if (args.Length > 1) {
                try {
                    iSleepTime = Convert.ToInt32(args[1]);
                }
                catch (FormatException) {
                    Console.WriteLine("待機時間の指定が不正です。ミリ秒単位で1以上の数を指定してください。");
                    return 1;
                }
            }

            // 文字間ウェイト取得
            if (args.Length > 2) {
                try {
                    iWaitTime = Convert.ToInt32(args[2]);
                }
                catch(FormatException) {
                    Console.WriteLine("文字間ウェイトの指定が不正です。ミリ秒単位で1以上の数を指定してください。");
                    return 1;
                }
            }

            // 入力対象のプログラムをアクティブにするまでの待ち
            Console.WriteLine("入力対象のプログラムをアクティブにしてください。");

	        Console.WriteLine(iSleepTime.ToString() + "ミリ秒間待機します...");
	        Thread.Sleep(iSleepTime);
	        Console.WriteLine("\n入力中...");
        	
	        // コマンドラインの文字列を取得する
            string p = args[0];

	        // 取得された文字列を１文字ずつ検査し、
	        // シミュレート可能文字列であればkeybd_event() APIに
	        // 渡す。
            foreach (char pi in p) {
		        // 入力対象の文字列かどうかを調べる
		        if ( checkKey(pi) ) {
			        // キーの押し下げをシミュレートする。
                    win32api.keybd_event((byte)pi, 0, 0, (UIntPtr)0);
			        // キーの解放をシミュレートする。
                    win32api.keybd_event((byte)pi, 0, 2/*KEYEVENTF_KEYUP*/, (UIntPtr)0);

			        // dwWaitTimeミリ秒間待機する
			        // (キーの取りこぼしを防ぐため)
                    Thread.Sleep(iWaitTime);
		        }
	        }

	        Console.WriteLine("キーボードシミュレートが終了しました。");

            return 0;
        }

        // ********************************************************************
        // * checkKey                                                         *
        // *   キーボードシミュレート対象文字列かどうかを調べる。             *
        // *   引数cが対象ならばtrueを返し、そうでなければfalseを返す。       *
        // ********************************************************************
        public static bool checkKey(char c)
        {
            if ( '0' <= c && c <= '9' ) { // 数字ならＯＫ
	            return true;
            }
            else if ( 'A' <= c && c <= 'Z' ) { // 英大文字ならＯＫ
	            return true;
            }
            else if ( c == ' ' ) { // スペースならＯＫ
	            return true;
            }
            else if ( c == '\t' ) { // タブならＯＫ
	            return true;
            }

            // それら以外の場合はＮＧ
            return false;
        }
    }
}
